@@ -3,13 +3,13 @@ package Net::DNS::Parameters;
 ################################################
 ##
 ##	Domain Name System (DNS) Parameters
-##	(last updated 2022-12-06)
+##	(last updated 2025-12-29)
 ##
 ################################################
 
 use strict;
 use warnings;
-our $VERSION = (qw$Id: Parameters.pm 1886 2022-12-20 11:27:25Z willem $)[2];
+our $VERSION = (qw$Id: Parameters.pm 2043 2026-01-14 13:35:59Z willem $)[2];
 
 use integer;
 use Carp;
@@ -50,6 +50,9 @@ our %classbyname = ( '*' => 255, @classbyname );
 
 # Registry: Resource Record (RR) TYPEs
 my @typebyname = (
+	DELEG	   => 65432,					# draft-ietf-deleg-02
+	DELEG	   => 61440,					# draft-ietf-deleg-03
+	DELEGI	   => 65433,					# draft-ietf-deleg-03
 	A	   => 1,					# RFC1035
 	NS	   => 2,					# RFC1035
 	MD	   => 3,					# RFC1035
@@ -71,8 +74,8 @@ my @typebyname = (
 	X25	   => 19,					# RFC1183
 	ISDN	   => 20,					# RFC1183
 	RT	   => 21,					# RFC1183
-	NSAP	   => 22,					# RFC1706
-	'NSAP-PTR' => 23,					# RFC1706
+	NSAP	   => 22,					# RFC1706 https://datatracker.ietf.org/doc/status-change-int-tlds-to-historic
+	'NSAP-PTR' => 23,					# RFC1706 https://datatracker.ietf.org/doc/status-change-int-tlds-to-historic
 	SIG	   => 24,					# RFC2536 RFC2931 RFC3110 RFC4034
 	KEY	   => 25,					# RFC2536 RFC2539 RFC3110 RFC4034
 	PX	   => 26,					# RFC2163
@@ -89,7 +92,7 @@ my @typebyname = (
 	CERT	   => 37,					# RFC4398
 	A6	   => 38,					# RFC2874 RFC3226 RFC6563
 	DNAME	   => 39,					# RFC6672
-	SINK	   => 40,					# draft-eastlake-kitchen-sink
+	SINK	   => 40,					# draft-eastlake-kitchen-sink-02
 	OPT	   => 41,					# RFC3225 RFC6891
 	APL	   => 42,					# RFC3123
 	DS	   => 43,					# RFC4034
@@ -112,8 +115,11 @@ my @typebyname = (
 	OPENPGPKEY => 61,					# RFC7929
 	CSYNC	   => 62,					# RFC7477
 	ZONEMD	   => 63,					# RFC8976
-	SVCB	   => 64,					# RFC-ietf-dnsop-svcb-https-10
-	HTTPS	   => 65,					# RFC-ietf-dnsop-svcb-https-10
+	SVCB	   => 64,					# RFC9460
+	HTTPS	   => 65,					# RFC9460
+	DSYNC	   => 66,					# RFC9859
+	HHIT	   => 67,					# RFC9886
+	BRID	   => 68,					# RFC9886
 	SPF	   => 99,					# RFC7208
 	UINFO	   => 100,					# IANA-Reserved
 	UID	   => 101,					# IANA-Reserved
@@ -125,6 +131,7 @@ my @typebyname = (
 	LP	   => 107,					# RFC6742
 	EUI48	   => 108,					# RFC7043
 	EUI64	   => 109,					# RFC7043
+	NXNAME	   => 128,					# RFC9824
 	TKEY	   => 249,					# RFC2930
 	TSIG	   => 250,					# RFC8945
 	IXFR	   => 251,					# RFC1995
@@ -135,9 +142,13 @@ my @typebyname = (
 	URI	   => 256,					# RFC7553
 	CAA	   => 257,					# RFC8659
 	AVC	   => 258,					#
-	DOA	   => 259,					# draft-durand-doa-over-dns
+	DOA	   => 259,					# draft-durand-doa-over-dns-02
 	AMTRELAY   => 260,					# RFC8777
-	TA	   => 32768,					# http://cameo.library.cmu.edu/ http://www.watson.org/~weiler/INI1999-19.pdf
+	RESINFO	   => 261,					# RFC9606
+	WALLET	   => 262,					#
+	CLA	   => 263,					# draft-johnson-dns-ipn-cla-07
+	IPN	   => 264,					# draft-johnson-dns-ipn-cla-07
+	TA	   => 32768,					# http://www.watson.org/~weiler/INI1999-19.pdf
 	DLV	   => 32769,					# RFC8749 RFC4431
 	);
 our %typebyval = reverse( TYPE0 => 0, @typebyname );
@@ -191,24 +202,28 @@ our %rcodebyname = @rcodebyname;
 
 # Registry: DNS EDNS0 Option Codes (OPT)
 my @ednsoptionbyname = (
-	LLQ		 => 1,					# RFC8764
-	UL		 => 2,					# http://files.dns-sd.org/draft-sekar-dns-ul.txt
-	NSID		 => 3,					# RFC5001
-	DAU		 => 5,					# RFC6975
-	DHU		 => 6,					# RFC6975
-	N3U		 => 7,					# RFC6975
-	'CLIENT-SUBNET'	 => 8,					# RFC7871
-	EXPIRE		 => 9,					# RFC7314
-	COOKIE		 => 10,					# RFC7873
-	'TCP-KEEPALIVE'	 => 11,					# RFC7828
-	PADDING		 => 12,					# RFC7830
-	CHAIN		 => 13,					# RFC7901
-	'KEY-TAG'	 => 14,					# RFC8145
-	'EXTENDED-ERROR' => 15,					# RFC8914
-	'CLIENT-TAG'	 => 16,					# draft-bellis-dnsop-edns-tags
-	'SERVER-TAG'	 => 17,					# draft-bellis-dnsop-edns-tags
-	'UMBRELLA-IDENT' => 20292,				# https://developer.cisco.com/docs/cloud-security/#!integrating-network-devic
-	DEVICEID	 => 26946,				# https://developer.cisco.com/docs/cloud-security/#!network-devices-getting-s
+	LLQ		  => 1,					# RFC8764
+	'UPDATE-LEASE'	  => 2,					# RFC9664
+	NSID		  => 3,					# RFC5001
+	DAU		  => 5,					# RFC6975
+	DHU		  => 6,					# RFC6975
+	N3U		  => 7,					# RFC6975
+	'CLIENT-SUBNET'	  => 8,					# RFC7871
+	EXPIRE		  => 9,					# RFC7314
+	COOKIE		  => 10,				# RFC7873
+	'TCP-KEEPALIVE'	  => 11,				# RFC7828
+	PADDING		  => 12,				# RFC7830
+	CHAIN		  => 13,				# RFC7901
+	'KEY-TAG'	  => 14,				# RFC8145
+	'EXTENDED-ERROR'  => 15,				# RFC8914
+	'CLIENT-TAG'	  => 16,				# draft-bellis-dnsop-edns-tags-01
+	'SERVER-TAG'	  => 17,				# draft-bellis-dnsop-edns-tags-01
+	'REPORT-CHANNEL'  => 18,				# RFC9567
+	ZONEVERSION	  => 19,				# RFC9660
+	'MQTYPE-QUERY'	  => 20,				# draft-ietf-dnssd-multi-qtypes-07
+	'MQTYPE-RESPONSE' => 21,				# draft-ietf-dnssd-multi-qtypes-07
+	'UMBRELLA-IDENT'  => 20292,				# https://developer.cisco.com/docs/cloud-security/#!integrating-network-devic
+	DEVICEID	  => 26946,				# https://developer.cisco.com/docs/cloud-security/#!network-devices-getting-s
 	);
 our %ednsoptionbyval = reverse @ednsoptionbyname;
 push @ednsoptionbyname, map { /^\d/ ? $_ : lc($_) } @ednsoptionbyname;
@@ -231,6 +246,7 @@ our %dnsflagbyname = @dnsflagbyname;
 # Registry: EDNS Header Flags (16 bits)
 my @ednsflagbyname = (
 	DO => 0x8000,						# RFC4035 RFC3225 RFC6840
+	CO => 0x4000,						# RFC9824
 	);
 push @ednsflagbyname, map { /^\d/ ? $_ : lc($_) } @ednsflagbyname;
 our %ednsflagbyname = @ednsflagbyname;
@@ -281,7 +297,9 @@ my @dnserrorbyval = (
 	25 => 'Signature Expired before Valid',			# https://github.com/NLnetLabs/unbound/pull/604#discussion_r802678343
 	26 => 'Too Early',					# RFC9250
 	27 => 'Unsupported NSEC3 Iterations Value',		# RFC9276
-	28 => 'Unable to conform to policy',			#
+	28 => 'Unable to conform to policy',			# draft-homburg-dnsop-codcp-00
+	29 => 'Synthesized',					# https://github.com/PowerDNS/pdns/pull/12334
+	30 => 'Invalid Query Type',				# RFC9824
 	);
 our %dnserrorbyval = @dnserrorbyval;
 
@@ -296,18 +314,18 @@ sub classbyname {
 	return $classbyname{$name} || $classbyname{uc $name} || return do {
 		croak qq[unknown class "$name"] unless $name =~ m/^(CLASS)?(\d+)/i;
 		my $val = 0 + $2;
-		croak qq[classbyname("$name") out of range] if $val > 0xffff;
+		croak qq[classbyname("$name") out of range] if $val > 0x7fff;
 		return $val;
 	}
 }
 
 sub classbyval {
-	my $val = shift;
+	my $arg = shift;
 
-	return $classbyval{$val} || return do {
-		$val += 0;
-		croak qq[classbyval($val) out of range] if $val > 0xffff;
-		return "CLASS$val";
+	return $classbyval{$arg} || return do {
+		my $val = ( $arg += 0 ) & 0x7fff;		# MSB used by mDNS
+		croak qq[classbyval($arg) out of range] if $arg > 0xffff;
+		return $classbyval{$arg} = $classbyval{$val} || "CLASS$val";
 	}
 }
 
@@ -399,9 +417,9 @@ use constant EXTLANG => defined eval { require Net::DNS::Extlang };
 
 sub _typespec {
 	my $generate = defined wantarray;
-	return EXTLANG ? eval <<'END' : '';			# no critic
-	my ($node) = @_;		## draft-levine-dnsextlang
-	my $instance = Net::DNS::Extlang->new();
+	return EXTLANG ? eval <<'END' : '';	## no critic
+	my ($node) = @_;
+	my $instance = Net::DNS::Extlang->new();	## draft-levine-dnsextlang
 	my $basename = $instance->domain || return '';
 
 	require Net::DNS::Resolver;
@@ -444,7 +462,7 @@ Net::DNS::Parameters - DNS parameter assignments
 
 =head1 SYNOPSIS
 
-    use Net::DNS::Parameters;
+	use Net::DNS::Parameters;
 
 
 =head1 DESCRIPTION
