@@ -2,7 +2,7 @@ package Net::DNS::RR::RRSIG;
 
 use strict;
 use warnings;
-our $VERSION = (qw$Id: RRSIG.pm 1819 2020-10-19 08:07:24Z willem $)[2];
+our $VERSION = (qw$Id: RRSIG.pm 2042 2025-12-24 10:23:11Z willem $)[2];
 
 use base qw(Net::DNS::RR);
 
@@ -26,38 +26,35 @@ use constant UTIL => defined eval { require Scalar::Util; };
 
 eval { require MIME::Base64 };
 
-# IMPORTANT: Downstream distros MUST NOT create dependencies on Net::DNS::SEC	(strong crypto prohibited in many territories)
-use constant USESEC => defined $INC{'Net/DNS/SEC.pm'};		# Discover how we got here, without exposing any crypto
-								# Discourage static code analysers and casual greppers
-use constant DNSSEC => USESEC && defined eval join '', qw(r e q u i r e), ' Net::DNS', qw(:: SEC :: Private);	## no critic
+## IMPORTANT: MUST NOT include crypto packages in metadata (strong crypto prohibited in many territories)
+use constant DNSSEC => defined $INC{'Net/DNS/SEC.pm'};	## Discover how we got here, without exposing any crypto
 
-my @index;
+my @algorithms;
+my @deprecated;
 if (DNSSEC) {
-	my $key = Net::DNS::RR->new( type => 'DNSKEY', key => 'AwEAAQ==' );
-	foreach my $class ( map {"Net::DNS::SEC::$_"} qw(RSA DSA ECCGOST ECDSA EdDSA) ) {
-		my @algorithms = eval join '', qw(r e q u i r e), " $class; $class->_index";	## no critic
-		@algorithms = grep { eval { $key->algorithm($_); $class->verify( '', $key, '' ); 1 } } ( 1 .. 16 )
-				unless scalar(@algorithms);	# Grotesquely inefficient; but need to support pre-1.14 API
-		push @index, map { ( $_ => $class ) } @algorithms;
+	foreach my $class ( map {"Net::DNS::SEC::$_"} qw(Private DSA RSA ECDSA EdDSA Digest SM2) ) {
+		my @index = eval join '', qw(r e q u i r e), " $class; ${class}::_index()";	## no critic
+		push @algorithms, map { ( $_ => $class ) } @index;
+		push @deprecated, eval "${class}::_deprecate()";			   	## no critic
 	}
+	croak 'Net::DNS::SEC version not supported' unless scalar(@algorithms);
 }
 
-my %DNSSEC_verify = @index;
-my %DNSSEC_siggen = @index;
+my %DNSSEC_verify = @algorithms;
+my %DNSSEC_siggen = @algorithms;
 
-my @deprecated = ( 1, 3, 6, 12 );				# RFC8624
-delete @DNSSEC_siggen{@deprecated};
+delete @DNSSEC_verify{@deprecated};	## DNSSEC status per RFC9904
+delete @DNSSEC_siggen{map { abs($_) } @deprecated};
 
 my @field = qw(typecovered algorithm labels orgttl sigexpiration siginception keytag);
 
 
 sub _decode_rdata {			## decode rdata from wire-format octet string
-	my $self = shift;
-	my ( $data, $offset ) = @_;
+	my ( $self, $data, $offset, @opaque ) = @_;
 
 	my $limit = $offset + $self->{rdlength};
 	@{$self}{@field} = unpack "\@$offset n C2 N3 n", $$data;
-	( $self->{signame}, $offset ) = Net::DNS::DomainName->decode( $data, $offset + 18 );
+	( $self->{signame}, $offset ) = Net::DNS::DomainName->decode( $data, $offset + 18, @opaque );
 	$self->{sigbin} = substr $$data, $offset, $limit - $offset;
 	return;
 }
@@ -77,15 +74,16 @@ sub _format_rdata {			## format rdata portion of RR string.
 	my $signame = $self->{signame};
 	my @sig64   = split /\s+/, MIME::Base64::encode( $self->sigbin );
 	my @rdata   = ( map( { $self->$_ } @field ), $signame->string, @sig64 );
+	$rdata[3] .= "\n";
 	return @rdata;
 }
 
 
 sub _parse_rdata {			## populate RR from rdata in argument list
-	my $self = shift;
+	my ( $self, @argument ) = @_;
 
-	foreach ( @field, qw(signame) ) { $self->$_(shift) }
-	$self->signature(@_);
+	foreach ( @field, qw(signame) ) { $self->$_( shift @argument ) }
+	$self->signature(@argument);
 	return;
 }
 
@@ -98,60 +96,9 @@ sub _defaults {				## specify RR attribute default values
 }
 
 
-#
-# source: http://www.iana.org/assignments/dns-sec-alg-numbers
-#
-{
-	my @algbyname = (
-		'DELETE'	     => 0,			# [RFC4034][RFC4398][RFC8078]
-		'RSAMD5'	     => 1,			# [RFC3110][RFC4034]
-		'DH'		     => 2,			# [RFC2539]
-		'DSA'		     => 3,			# [RFC3755][RFC2536]
-					## Reserved	=> 4,	# [RFC6725]
-		'RSASHA1'	     => 5,			# [RFC3110][RFC4034]
-		'DSA-NSEC3-SHA1'     => 6,			# [RFC5155]
-		'RSASHA1-NSEC3-SHA1' => 7,			# [RFC5155]
-		'RSASHA256'	     => 8,			# [RFC5702]
-					## Reserved	=> 9,	# [RFC6725]
-		'RSASHA512'	     => 10,			# [RFC5702]
-					## Reserved	=> 11,	# [RFC6725]
-		'ECC-GOST'	     => 12,			# [RFC5933]
-		'ECDSAP256SHA256'    => 13,			# [RFC6605]
-		'ECDSAP384SHA384'    => 14,			# [RFC6605]
-		'ED25519'	     => 15,			# [RFC8080]
-		'ED448'		     => 16,			# [RFC8080]
-
-		'INDIRECT'   => 252,				# [RFC4034]
-		'PRIVATEDNS' => 253,				# [RFC4034]
-		'PRIVATEOID' => 254,				# [RFC4034]
-					## Reserved	=> 255,	# [RFC4034]
-		);
-
-	my %algbyval = reverse @algbyname;
-
-	foreach (@algbyname) { s/[\W_]//g; }			# strip non-alphanumerics
-	my @algrehash = map { /^\d/ ? ($_) x 3 : uc($_) } @algbyname;
-	my %algbyname = @algrehash;				# work around broken cperl
-
-	sub _algbyname {
-		my $arg = shift;
-		my $key = uc $arg;				# synthetic key
-		$key =~ s/[\W_]//g;				# strip non-alphanumerics
-		my $val = $algbyname{$key};
-		return $val if defined $val;
-		return $key =~ /^\d/ ? $arg : croak qq[unknown algorithm "$arg"];
-	}
-
-	sub _algbyval {
-		my $value = shift;
-		return $algbyval{$value} || return $value;
-	}
-}
-
-
 sub typecovered {
-	my $self = shift;
-	$self->{typecovered} = typebyname(shift) if scalar @_;
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{typecovered} = typebyname($_) }
 	my $typecode = $self->{typecovered};
 	return defined $typecode ? typebyval($typecode) : undef;
 }
@@ -172,32 +119,30 @@ sub algorithm {
 
 
 sub labels {
-	my $self = shift;
-
-	$self->{labels} = 0 + shift if scalar @_;
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{labels} = 0 + $_ }
 	return $self->{labels} || 0;
 }
 
 
 sub orgttl {
-	my $self = shift;
-
-	$self->{orgttl} = 0 + shift if scalar @_;
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{orgttl} = 0 + $_ }
 	return $self->{orgttl} || 0;
 }
 
 
 sub sigexpiration {
-	my $self = shift;
-	$self->{sigexpiration} = _string2time(shift) if scalar @_;
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{sigexpiration} = _string2time($_) }
 	my $time = $self->{sigexpiration};
 	return unless defined wantarray && defined $time;
 	return UTIL ? Scalar::Util::dualvar( $time, _time2string($time) ) : _time2string($time);
 }
 
 sub siginception {
-	my $self = shift;
-	$self->{siginception} = _string2time(shift) if scalar @_;
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{siginception} = _string2time($_) }
 	my $time = $self->{siginception};
 	return unless defined wantarray && defined $time;
 	return UTIL ? Scalar::Util::dualvar( $time, _time2string($time) ) : _time2string($time);
@@ -208,39 +153,36 @@ sub sigex { return &sigexpiration; }	## historical
 sub sigin { return &siginception; }	## historical
 
 sub sigval {
-	my $self = shift;
+	my ( $self, @value ) = @_;
 	no integer;
-	return ( $self->{sigval} ) = map { int( 86400 * $_ ) } @_;
+	return ( $self->{sigval} ) = map { int( 86400 * $_ ) } @value;
 }
 
 
 sub keytag {
-	my $self = shift;
-
-	$self->{keytag} = 0 + shift if scalar @_;
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{keytag} = 0 + $_ }
 	return $self->{keytag} || 0;
 }
 
 
 sub signame {
-	my $self = shift;
-
-	$self->{signame} = Net::DNS::DomainName->new(shift) if scalar @_;
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{signame} = Net::DNS::DomainName->new($_) }
 	return $self->{signame} ? $self->{signame}->name : undef;
 }
 
 
 sub sig {
-	my $self = shift;
-	return MIME::Base64::encode( $self->sigbin(), "" ) unless scalar @_;
-	return $self->sigbin( MIME::Base64::decode( join "", @_ ) );
+	my ( $self, @value ) = @_;
+	return MIME::Base64::encode( $self->sigbin(), "" ) unless scalar @value;
+	return $self->sigbin( MIME::Base64::decode( join "", @value ) );
 }
 
 
 sub sigbin {
-	my $self = shift;
-
-	$self->{sigbin} = shift if scalar @_;
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{sigbin} = $_ }
 	return $self->{sigbin} || "";
 }
 
@@ -390,6 +332,56 @@ sub vrfyerrstr {
 
 ########################################
 
+{
+	my @algbyname = (
+		'DELETE'	     => 0,			# [RFC4034][RFC4398][RFC8078]
+		'RSAMD5'	     => 1,			# [RFC3110][RFC4034]
+		'DH'		     => 2,			# [RFC2539]
+		'DSA'		     => 3,			# [RFC3755][RFC2536]
+					## Reserved	=> 4,	# [RFC6725]
+		'RSASHA1'	     => 5,			# [RFC3110][RFC4034]
+		'DSA-NSEC3-SHA1'     => 6,			# [RFC5155]
+		'RSASHA1-NSEC3-SHA1' => 7,			# [RFC5155]
+		'RSASHA256'	     => 8,			# [RFC5702]
+					## Reserved	=> 9,	# [RFC6725]
+		'RSASHA512'	     => 10,			# [RFC5702]
+					## Reserved	=> 11,	# [RFC6725]
+		'ECC-GOST'	     => 12,			# [RFC5933]
+		'ECDSAP256SHA256'    => 13,			# [RFC6605]
+		'ECDSAP384SHA384'    => 14,			# [RFC6605]
+		'ED25519'	     => 15,			# [RFC8080]
+		'ED448'		     => 16,			# [RFC8080]
+		'SM2SM3'	     => 17,			# [RFC9563]
+		'ECC-GOST12'	     => 23,			# [RFC9558]
+
+		'INDIRECT'   => 252,				# [RFC4034]
+		'PRIVATEDNS' => 253,				# [RFC4034]
+		'PRIVATEOID' => 254,				# [RFC4034]
+					## Reserved	=> 255,	# [RFC4034]
+		);
+
+	my %algbyval = reverse @algbyname;
+
+	foreach (@algbyname) { s/[\W_]//g; }			# strip non-alphanumerics
+	my @algrehash = map { /^\d/ ? ($_) x 3 : uc($_) } @algbyname;
+	my %algbyname = @algrehash;				# work around broken cperl
+
+	sub _algbyname {
+		my $arg = shift;
+		my $key = uc $arg;				# synthetic key
+		$key =~ s/[\W_]//g;				# strip non-alphanumerics
+		my $val = $algbyname{$key};
+		return $val if defined $val;
+		return $key =~ /^\d/ ? $arg : croak qq[unknown algorithm $arg];
+	}
+
+	sub _algbyval {
+		my $value = shift;
+		return $algbyval{$value} || return $value;
+	}
+}
+
+
 sub _CreateSigData {
 
 	# This method creates the data string that will be signed.
@@ -461,18 +453,15 @@ sub _CreateSigData {
 }
 
 
-########################################
-
 sub _CreateSig {
 	if (DNSSEC) {
-		my $self = shift;
+		my ( $self, @argument ) = @_;
 
 		my $algorithm = $self->algorithm;
-		my $class     = $DNSSEC_siggen{$algorithm};
-
 		return eval {
+			my $class = $DNSSEC_siggen{$algorithm};
 			die "algorithm $algorithm not supported\n" unless $class;
-			$self->sigbin( $class->sign(@_) );
+			$self->sigbin( $class->sign(@argument) );
 		} || return croak "${@}signature generation failed";
 	}
 }
@@ -480,42 +469,39 @@ sub _CreateSig {
 
 sub _VerifySig {
 	if (DNSSEC) {
-		my $self = shift;
+		my ( $self, @argument ) = @_;
 
 		my $algorithm = $self->algorithm;
-		my $class     = $DNSSEC_verify{$algorithm};
-
-		my $retval = eval {
+		my $returnval = eval {
+			my $class = $DNSSEC_verify{$algorithm};
 			die "algorithm $algorithm not supported\n" unless $class;
-			$class->verify( @_, $self->sigbin );
+			$class->verify( @argument, $self->sigbin );
 		};
 
-		unless ($retval) {
+		unless ($returnval) {
 			$self->{vrfyerrstr} = "${@}signature verification failed";
 			print "\n", $self->{vrfyerrstr}, "\n" if DEBUG;
 			return 0;
 		}
 
-		# uncoverable branch true	# bug in Net::DNS::SEC or dependencies
-		croak "unknown error in $class->verify" unless $retval == 1;
+		# uncoverable branch true	# unexpected return value from EVP_DigestVerify
+		croak "internal error in algorithm $algorithm verification" unless $returnval == 1;
 		print "\nalgorithm $algorithm verification successful\n" if DEBUG;
-		return 1;
+		return $returnval;
 	}
 }
 
 
 sub _ordered() {			## irreflexive 32-bit partial ordering
-	use integer;
 	my ( $n1, $n2 ) = @_;
 
 	return 0 unless defined $n2;				# ( any, undef )
 	return 1 unless defined $n1;				# ( undef, any )
 
 	# unwise to assume 64-bit arithmetic, or that 32-bit integer overflow goes unpunished
-	if ( $n2 < 0 ) {					# fold, leaving $n2 non-negative
-		$n1 = ( $n1 & 0xFFFFFFFF ) ^ 0x80000000;	# -2**31 <= $n1 < 2**32
-		$n2 = ( $n2 & 0x7FFFFFFF );			#  0	 <= $n2 < 2**31
-	}
+	use integer;						# fold, leaving $n2 non-negative
+	$n1 = ( $n1 & 0xFFFFFFFF ) ^ ( $n2 & 0x80000000 );	# -2**31 <= $n1 < 2**32
+	$n2 = ( $n2 & 0x7FFFFFFF );				#  0	 <= $n2 < 2**31
 
 	return $n1 < $n2 ? ( $n1 > ( $n2 - 0x80000000 ) ) : ( $n2 < ( $n1 - 0x80000000 ) );
 }
@@ -569,6 +555,8 @@ sub _time2string {			## format time specification string
 	return sprintf '%d%02d%02d%02d%02d%02d', $yy + 1900, $mm + 1, @dhms;
 }
 
+########################################
+
 
 1;
 __END__
@@ -576,18 +564,19 @@ __END__
 
 =head1 SYNOPSIS
 
-    use Net::DNS;
-    $rr = Net::DNS::RR->new('name RRSIG typecovered algorithm labels
+	use Net::DNS;
+	$rr = Net::DNS::RR->new('name RRSIG typecovered algorithm labels
 				orgttl sigexpiration siginception
 				keytag signame signature');
 
-    use Net::DNS::SEC;
-    $sigrr = Net::DNS::RR::RRSIG->create( \@rrset, $keypath,
-					sigex => 20191231010101
-					sigin => 20191201010101
-					);
+	use Net::DNS::SEC;
+	$sigrr = Net::DNS::RR::RRSIG->create(
+				\@rrset, $keypath,
+				sigex => 20251230010101,
+				sigin => 20251201010101
+				);
 
-    $sigrr->verify( \@rrset, $keyrr ) || die $sigrr->vrfyerrstr;
+	$sigrr->verify( \@rrset, $keyrr ) || die $sigrr->vrfyerrstr;
 
 =head1 DESCRIPTION
 
@@ -612,14 +601,14 @@ other unpredictable behaviour.
 
 =head2 typecovered
 
-    $typecovered = $rr->typecovered;
+	$typecovered = $rr->typecovered;
 
 The typecovered field identifies the type of the RRset that is
 covered by this RRSIG record.
 
 =head2 algorithm
 
-    $algorithm = $rr->algorithm;
+	$algorithm = $rr->algorithm;
 
 The algorithm number field identifies the cryptographic algorithm
 used to create the signature.
@@ -629,16 +618,16 @@ to perform mnemonic and numeric code translation.
 
 =head2 labels
 
-    $labels = $rr->labels;
-    $rr->labels( $labels );
+	$labels = $rr->labels;
+	$rr->labels( $labels );
 
 The labels field specifies the number of labels in the original RRSIG
 RR owner name.
 
 =head2 orgttl
 
-    $orgttl = $rr->orgttl;
-    $rr->orgttl( $orgttl );
+	$orgttl = $rr->orgttl;
+	$rr->orgttl( $orgttl );
 
 The original TTL field specifies the TTL of the covered RRset as it
 appears in the authoritative zone.
@@ -647,11 +636,11 @@ appears in the authoritative zone.
 
 =head2 sigex sigin sigval
 
-    $expiration = $rr->sigexpiration;
-    $expiration = $rr->sigexpiration( $value );
+	$expiration = $rr->sigexpiration;
+	$expiration = $rr->sigexpiration( $value );
 
-    $inception = $rr->siginception;
-    $inception = $rr->siginception( $value );
+	$inception = $rr->siginception;
+	$inception = $rr->siginception( $value );
 
 The signature expiration and inception fields specify a validity
 time interval for the signature.
@@ -664,16 +653,16 @@ numerical Perl time() value.
 
 =head2 keytag
 
-    $keytag = $rr->keytag;
-    $rr->keytag( $keytag );
+	$keytag = $rr->keytag;
+	$rr->keytag( $keytag );
 
 The keytag field contains the key tag value of the DNSKEY RR that
 validates this signature.
 
 =head2 signame
 
-    $signame = $rr->signame;
-    $rr->signame( $signame );
+	$signame = $rr->signame;
+	$rr->signame( $signame );
 
 The signer name field value identifies the owner name of the DNSKEY
 RR that a validator is supposed to use to validate this signature.
@@ -682,8 +671,8 @@ RR that a validator is supposed to use to validate this signature.
 
 =head2 sig
 
-    $sig = $rr->sig;
-    $rr->sig( $sig );
+	$sig = $rr->sig;
+	$rr->sig( $sig );
 
 The Signature field contains the cryptographic signature that covers
 the RRSIG RDATA (excluding the Signature field) and the RRset
@@ -692,8 +681,8 @@ covered fields.
 
 =head2 sigbin
 
-    $sigbin = $rr->sigbin;
-    $rr->sigbin( $sigbin );
+	$sigbin = $rr->sigbin;
+	$rr->sigbin( $sigbin );
 
 Binary representation of the cryptographic signature.
 
@@ -701,24 +690,25 @@ Binary representation of the cryptographic signature.
 
 Create a signature over a RR set.
 
-    use Net::DNS::SEC;
+	use Net::DNS::SEC;
 
-    $keypath = '/home/olaf/keys/Kbla.foo.+001+60114.private';
+	$keypath = '/home/olaf/keys/Kbla.foo.+001+60114.private';
 
-    $sigrr = Net::DNS::RR::RRSIG->create( \@rrsetref, $keypath );
+	$sigrr = Net::DNS::RR::RRSIG->create( \@rrsetref, $keypath );
 
-    $sigrr = Net::DNS::RR::RRSIG->create( \@rrsetref, $keypath,
-					sigex => 20191231010101
-					sigin => 20191201010101
-					);
-    $sigrr->print;
+	$sigrr = Net::DNS::RR::RRSIG->create(
+				\@rrsetref, $keypath,
+				sigex => 20251230010101,
+				sigin => 20251201010101
+				);
+	$sigrr->print;
 
 
-    # Alternatively use Net::DNS::SEC::Private 
+	# Alternatively use Net::DNS::SEC::Private 
 
-    $private = Net::DNS::SEC::Private->new($keypath);
+	$private = Net::DNS::SEC::Private->new($keypath);
 
-    $sigrr= Net::DNS::RR::RRSIG->create( \@rrsetref, $private );
+	$sigrr= Net::DNS::RR::RRSIG->create( \@rrsetref, $private );
 
 
 create() is an alternative constructor for a RRSIG RR object.  
@@ -735,10 +725,10 @@ containing the private key as generated by dnssec-keygen.
 The optional remaining arguments consist of ( name => value ) pairs
 as follows:
 
-	sigex  => 20191231010101,	# signature expiration
-	sigin  => 20191201010101,	# signature inception
-	sigval => 30,			# validity window (days)
-	ttl    => 3600			# TTL
+	sigex	=> 20251230010101,	# signature expiration
+	sigin	=> 20251201010101,	# signature inception
+	sigval	=> 30,			# validity window (days)
+	ttl	=> 3600
 
 The sigin and sigex values may be specified as Perl time values or as
 a string with the format 'yyyymmddhhmmss'. The default for sigin is
@@ -753,8 +743,8 @@ By default the TTL matches the RRset that is presented for signing.
 
 =head2 verify
 
-    $verify = $sigrr->verify( $rrsetref, $keyrr );
-    $verify = $sigrr->verify( $rrsetref, [$keyrr, $keyrr2, $keyrr3] );
+	$verify = $sigrr->verify( $rrsetref, $keyrr );
+	$verify = $sigrr->verify( $rrsetref, [$keyrr, $keyrr2, $keyrr3] );
 
 $rrsetref contains a reference to an array of RR objects and the
 method verifies the RRset against the signature contained in the
@@ -769,10 +759,10 @@ Returns 0 on error and sets $sig->vrfyerrstr
 
 =head2 vrfyerrstr
 
-    $verify = $sigrr->verify( $rrsetref, $keyrr );
-    print $sigrr->vrfyerrstr unless $verify;
+	$verify = $sigrr->verify( $rrsetref, $keyrr );
+	print $sigrr->vrfyerrstr unless $verify;
 
-    $sigrr->verify( $rrsetref, $keyrr ) || die $sigrr->vrfyerrstr;
+	$sigrr->verify( $rrsetref, $keyrr ) || die $sigrr->vrfyerrstr;
 
 =head1 KEY GENERATION
 
@@ -780,11 +770,10 @@ Private key files and corresponding public DNSKEY records
 are most conveniently generated using dnssec-keygen,
 a program that comes with the ISC BIND distribution.
 
-    dnssec-keygen -a 10 -b 2048 -f ksk	rsa.example.
-    dnssec-keygen -a 10 -b 1024		rsa.example.
+	dnssec-keygen -a 10 -b 2048 rsa.example.
 
-    dnssec-keygen -a 14	-f ksk	ecdsa.example.
-    dnssec-keygen -a 14		ecdsa.example.
+	dnssec-keygen -a 13 -f ksk ecdsa.example.
+	dnssec-keygen -a 13	   ecdsa.example.
 
 Do not change the name of the private key file.
 The create method uses the filename as generated by dnssec-keygen
@@ -812,9 +801,6 @@ T.J. Mather provided support for the DSA algorithm.
 
 Dick Franks added support for elliptic curve and Edwards curve algorithms.
 
-Mike McCauley created the Crypt::OpenSSL::ECDSA perl extension module
-specifically for this development.
-
 
 =head1 COPYRIGHT
 
@@ -822,7 +808,7 @@ Copyright (c)2001-2005 RIPE NCC,   Olaf M. Kolkman
 
 Copyright (c)2007-2008 NLnet Labs, Olaf M. Kolkman
 
-Portions Copyright (c)2014 Dick Franks 
+Portions Copyright (c)2014,2025 Dick Franks 
 
 All rights reserved.
 
@@ -833,7 +819,7 @@ Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -850,11 +836,12 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, L<Net::DNS::SEC>,
-RFC4034, RFC6840, RFC3755
+L<perl> L<Net::DNS> L<Net::DNS::RR>
+L<Net::DNS::SEC>
+L<RFC4034(3)|https://iana.org/go/rfc4034#section-3>
 
-L<Algorithm Numbers|http://www.iana.org/assignments/dns-sec-alg-numbers>
+L<Algorithm Numbers|https://iana.org/assignments/dns-sec-alg-numbers>
 
-L<BIND 9 Administrator Reference Manual|http://www.bind9.net/manuals>
+L<BIND Administrator Reference Manual|https://bind9.readthedocs.io/en/latest/>
 
 =cut
